@@ -1,6 +1,7 @@
 from abc import ABC as _ABC, abstractmethod as _abstractmethod
 from asyncio import gather as _gather
 from datetime import date as _date, datetime as _datetime
+from enum import IntEnum as _IntEnum
 from io import StringIO as _StringIO
 from json import loads as _loads
 from logging import (
@@ -14,7 +15,7 @@ from re import (
 from typing import Any as _Any, TypedDict as _TypedDict
 
 import pandas as _pd
-from jdatetime import datetime as _jdatetime
+from jdatetime import date as _jdate, datetime as _jdatetime
 from pandas import (
     DataFrame as _DataFrame,
     Series as _Series,
@@ -78,7 +79,7 @@ def _fanum_to_num(series: _Series):
     Converts a pandas Series from strings containing Persian digits and commas
     to a numeric data type.
     """
-    return _to_numeric(series.str.translate(_fa_to_en_tt))  # type: ignore
+    return _to_numeric(series.str.translate(_fa_to_en_tt))
 
 
 class BaseSite(_ABC):
@@ -380,6 +381,28 @@ class _RHNavLight(_TypedDict):
     Time: str
 
 
+class FundList(_TypedDict):
+    FundId: int
+    FundName: str
+    IsDefaultFund: bool
+
+
+class FundType(_IntEnum):
+    # the values are defined in the first line of public.min.js e.g. in
+    # https://tazmin.charismafunds.ir/bundles/js/public.min.js?v=202508170532
+    # fundType={simple:1,simpleETF:2,hybrid:3,multiFund:4,multiETF:5};
+    SIMPLE = 1
+    SIMPLE_ETF = 2
+    HYBRID = 3
+    MULTI_FUND = 4
+    MULTI_ETF = 5
+
+
+class FundData(_TypedDict):
+    FundType: FundType
+    FundList: list[FundList]
+
+
 class RayanHamafza(BaseSite):
     _api_path = 'api/data'
     __slots__ = 'fund_id'
@@ -455,6 +478,7 @@ class RayanHamafza(BaseSite):
         df['ProfitDate'] = df['ProfitDate'].apply(
             lambda i: _jdatetime.strptime(i, format='%Y/%m/%d').togregorian()
         )
+        df.set_index('ProfitDate', inplace=True)
         return df
 
     async def cache(self) -> float:
@@ -465,6 +489,11 @@ class RayanHamafza(BaseSite):
             + aa['BondTodayPercent']
         )
 
+    async def fund_data(self) -> FundData:
+        fund_data = await self._json('Fund')
+        fund_data['FundType'] = FundType(fund_data['FundType'])
+        return fund_data
+
 
 _jp = _jdatetime.strptime
 
@@ -675,11 +704,35 @@ class TadbirPardaz(BaseTadbirPardaz):
         df.set_index('date', inplace=True)
         return df
 
-    async def dividend_history(self) -> _DataFrame:
-        path = 'Reports/FundDividendProfitReport'
+    async def dividend_history(
+        self,
+        *,
+        from_date: _date | str | None = None,
+        to_date: _date | str | None = None,
+    ) -> _DataFrame:
+        params: dict = {'page': 1}
+        if from_date is not None or to_date is not None:
+            if from_date is not None:
+                if isinstance(from_date, _date):
+                    jd = _jdate.fromgregorian(date=from_date)
+                    from_date = f'{jd.year}/{jd.month}/{jd.day}'
+                params['fromDate'] = from_date
+            if to_date is not None:
+                if isinstance(to_date, _date):
+                    jd = _jdate.fromgregorian(date=to_date)
+                    to_date = f'{jd.year}/{jd.month}/{jd.day}'
+                params['toDate'] = to_date
+
         all_rows = []
-        while path:
-            html = (await _read(f'{self.url}{path}')).decode()
+        while True:
+            html = (
+                await (
+                    await _get(
+                        f'{self.url}Reports/FundDividendProfitReport',
+                        params=params,
+                    )
+                ).read()
+            ).decode()
             table, _, after_table = html.partition('<tbody>')[2].rpartition(
                 '</tbody>'
             )
@@ -687,9 +740,13 @@ class TadbirPardaz(BaseTadbirPardaz):
                 _findall(r'<td>([^<]*)</td>', r)
                 for r in _split(r'</tr>\s*<tr>', table)
             ]
-            path = after_table.rpartition('" title="Next page">')[
-                0
-            ].rpartition('<a href="/')[2]
+            if '" title="Next page">' not in after_table:
+                break
+            params['page'] += 1
+
+        if not all_rows[0]:  # no data for selected range
+            return _DataFrame()
+
         # try to use the same column names as RayanHamafza.dividend_history
         df = _DataFrame(
             all_rows,
@@ -707,7 +764,8 @@ class TadbirPardaz(BaseTadbirPardaz):
         df[comma_cols] = df[comma_cols].map(_comma_int)
         int_cols = ['row', 'UnitProfit']
         df[int_cols] = df[int_cols].map(_comma_int)
-        df['ProfitPercent'] = df['ProfitPercent'].astype(float)
+        df['ProfitPercent'] = df['ProfitPercent'].map(_comma_float)
+        df.set_index('ProfitDate', inplace=True)
         return df
 
 
